import re
import glob
import os
import sys

match_static_inline = re.compile(r'static inline (.*?)\n([\S\s][^;]*?){')

def generate_ffi(notcurses_dir):
    lines = []
    for file in glob.glob(os.path.join(notcurses_dir, "include", "notcurses", "*.h")):
        with open(file, 'r') as f:
            content = f.read()
            finds = re.findall(match_static_inline, content)
            for find in finds:
                lines.append(find[0] + ' ' + find[1] + ';')

    lines.sort()

    with open(os.path.join(notcurses_dir, "src", "libffi", "ffi.c"), 'w') as f:
        f.write("// Contains all inline functions in include/notcurses/*.h\n")
        f.write("// This file is auto generated from tools/generate_ffi.py\n")
        f.write("#include <notcurses/notcurses.h>\n")
        f.write("#include <notcurses/direct.h>\n\n")
        f.write("#include <notcurses/nckeys.h>\n\n")

        for line in lines:
            line = line.replace("RESTRICT", "restrict")
            f.write(line)
            f.write('\n')

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("usage: generate_ffi.py notcurses-dir")
    else:
        generate_ffi(sys.argv[1])



# SPDX-License-Identifier: Apache-2.0

# Copyright 2020, 2021 igo95862

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

from os import environ

from setuptools import Extension, setup

from sys import platform

if environ.get('CFLAGS') is None:
    if platform == "darwin":
        environ['CFLAGS'] = (
            "-Wextra -Wconversion -Wall")
    else:
        environ['CFLAGS'] = (
            "-Werror "
            "-Wextra -Wconversion -Wall")

if environ.get('LDFLAGS') is None:
    if platform == "darwin":
        environ['LDFLAGS'] = "-Wl,-all_load"
    else:
        environ['LDFLAGS'] = "-Wl,--no-as-needed"

setup(
    name="notcurses",
    version="3.0.17",
    packages=['notcurses'],
    ext_modules=[
        Extension(
            name='notcurses.notcurses',
            sources=[
                'notcurses/channels.c',
                'notcurses/context.c',
                'notcurses/functions.c',
                'notcurses/main.c',
                'notcurses/misc.c',
                'notcurses/plane.c',
            ],
            libraries=['notcurses'],
            language='c',
        ),
    ],
    author="Nick Black",
    author_email="nickblack@linux.com",
    description="Blingful TUI construction library (python bindings)",
    keywords="ncurses curses tui console graphics",
    license='Apache License, Version 2.0',
    url='https://github.com/dankamongmen/notcurses',
    zip_safe=True,
    # see https://pypi.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
    ],
    package_data={
        'notcurses': [
            'py.typed',
        ],
    }
)



# SPDX-License-Identifier: Apache-2.0

# Copyright 2020 igo95862

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Generator

from notcurses import NcChannels, NcDirect

# Acquire the NcDirect plane
nc_direct = NcDirect()
nc_direct.cursor_enabled = False
channels = NcChannels()
channels.set_background_rgb(255, 0, 0)
channels.set_foreground_rgb(0, 0, 0)

# Get x dimensions, ignore y
_, x_dimension = nc_direct.dimensions_yx

# Function to generate the green to red line


def red_line_gen(used_space: int = 0, percent: float = 1.0
                 ) -> Generator[int, None, None]:
    line_size = x_dimension - used_space  # How much space in the line we have
    blocks_to_put = round(percent * line_size)  # How many blocks to fill
    for x in range(blocks_to_put):
        # Yeilds the integer on what to reduce green and increase red by
        yield round(x * 255.0 / line_size)


# Open the meminfo file
with open('/proc/meminfo') as f:
    meminfo_lines = f.read().splitlines()


mem_total_line = meminfo_lines[0]  # Line 0 is the total memory
mem_avalible_line = meminfo_lines[2]

# The lines in meminfo file are like this
# MemTotal:        3801578 kB
# so we need to split by whitespace and get second item
mem_total = int(mem_total_line.split()[1])  # Get total memory
mem_avalible = int(mem_avalible_line.split()[1])  # Get avalible memory

mem_percent_used = 1.0 - mem_avalible / mem_total  # Calculate percent used

mem_sting = f"Memory used: {round(100.0 * mem_percent_used)}% "

nc_direct.putstr(mem_sting)  # Put the used memory

for red_shift in red_line_gen(len(mem_sting), mem_percent_used):
    # Get the red shift from the function and use it in red channel
    # and subtract it from green
    channels.set_background_rgb(red_shift, 255-red_shift, 0)
    channels.set_foreground_rgb(red_shift, 255-red_shift, 0)
    nc_direct.putstr('X', channels)

nc_direct.putstr('\n')  # Finish line

swap_total = int(meminfo_lines[14].split()[1])  # Get swap total
swap_avalible = int(meminfo_lines[15].split()[1])  # Get swap used

swap_percent_used = 1.0 - swap_avalible / swap_total

swap_string = f"Swap used:   {round(100.0 * swap_percent_used)}% "

# Add space in case swap is a single digit
if len(swap_string) < len(mem_sting):
    swap_string += ' '

nc_direct.putstr(swap_string)

for red_shift in red_line_gen(len(swap_string), swap_percent_used):
    channels.set_background_rgb(red_shift, 255-red_shift, 0)
    channels.set_foreground_rgb(red_shift, 255-red_shift, 0)
    nc_direct.putstr('X', channels)

nc_direct.putstr('\n')



from time import sleep
import notcurses as nc

notcurses = nc.Notcurses()
plane = notcurses.stdplane()

BOX_CHARS = (
    nc.NCBOXASCII,
    nc.NCBOXDOUBLE,
    nc.NCBOXHEAVY,
    nc.NCBOXLIGHT,
    nc.NCBOXOUTER,
    nc.NCBOXROUND,
)

COLORS = (
    (None, None),
    (None, nc.rgb(128, 128, 128)),          # default on grey
    (nc.rgb(255, 0, 0), None),              # red on default
    (nc.rgb(0, 255, 0), nc.rgb(0, 0, 255)), # green on blue
)

SY = 7
SX = 10

for y, (fg, bg) in enumerate(COLORS):
    for x, box_chars in enumerate(BOX_CHARS):
        plane.cursor_move_yx(y * SY + 1, x * SX + 1);
        nc.box(
            plane, (y + 1) * SY - 1, (x + 1) * SX - 1,
            box_chars,
            fg=fg, bg=bg,
            # ctlword=0x1f9
        )

notcurses.render()
sleep(5)



# SPDX-License-Identifier: Apache-2.0

# Copyright 2020 igo95862

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from notcurses import Notcurses

nc = Notcurses()
std_plane = nc.stdplane()
std_plane.putstr("Enter string!")

nc.render()
nc.mice_enable()

while True:
    # Get an input event and print its properties
    inp = nc.get_blocking()
    std_plane.erase()
    std_plane.putstr_yx(1, 4, f"Code point: {hex(inp.id)}")
    std_plane.putstr_yx(2, 4, f"Y pos: {inp.y}")
    std_plane.putstr_yx(3, 4, f"X pos: {inp.x}")
    std_plane.putstr_yx(4, 4, f"UTF-8: {inp.utf8!r}")
    std_plane.putstr_yx(5, 4, f"Event type: {inp.evtype}")
    std_plane.putstr_yx(6, 4, f"Modifiers: {bin(inp.modifiers)}")
    std_plane.putstr_yx(7, 4, f"Y pixel offset: {inp.ypx}")
    std_plane.putstr_yx(8, 4, f"X pixel offset: {inp.xpx}")
    std_plane.putstr_yx(10, 4, "Press CTRL+C to exit.")

    nc.render()



# SPDX-License-Identifier: Apache-2.0

# Copyright 2020 igo95862

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sys import path
from os.path import abspath

project = 'notcurses'
author = 'igo95862'
source_suffix = '.rst'
extensions = ['sphinx.ext.autodoc']

path.append(abspath('..'))
